<?php 

/*****************************************************

Phone Finder 3.0
Copyright 2010-2014 Jacob Strandlien

Intended for use with the following mobile tracking
softwares:
- Cerebro version 1.1 (Cydia)
  http://cerebro.ifindmyiphone.com/
- Device Locator (App Store)
  http://device-locator.com/
- Btraced (App Store, Google Play)
  http://www.btraced.com/
- Open GPS Tracker (Google Play)
- Self-Hosted GPS Tracker (Google Play)

This file acts both as a page collects the phone's (or
other device's) location data and the page that
displays it.  It also attempts some basic analysis of
the data, refining the position of a stationary phone
and eliminating coordinates grabbed from cell towers
in favor of more accurate GPS coordinates where
possible.  It displays this information in a
consolidated format, detecting when the phone is
stationary and displaying one refined set of
coordinates for the stay.  Multiple devices can be
tracked using the single script.

To use this script, create a MySQL database to store
the GPS data (you don't need to create any tables) and
put the database's login information into the first
three parameters of this script.  The script should
handle the rest.

Then all you need to do is to point the tracking
service's settings to this file (instructions in the
next two paragraphs).  If all goes well, it will
immediately start collecting the location data into
the MySQL database.  You will need to wait for the
database to gather at least three sets of coordinates
before the page will display properly.  Then just
point your web browser to this file to display the
information.

For Cerebro, just put the location of this file into
the "URL of recording" and "URL of Live Updating"
setting fields on the Cerebro website.

For Device Locator put the location of this file
followed directly by:
?LONGITUDE={long}&LATITUDE={lat}&ACCURACY={acc}&ALTITUDE={alt}&BATTERYLEVEL={batt}&ID=MyPhone$TIMEDIFF={timediff}
into the "Forwarding URL" setting field on the Device
Locator website.  You can replace MyPhone with any
text that you want to use to identify the individual
phone or other device.

For Open GPS Tracker, put the location of this file
followed directly by:
?APP=OGT&LONGITUDE=@LON@&LATITUDE=@LAT@&ACCURACY=@ACC@&SPEED=@SPEED@&BEARING=@BEAR@&TIME=@TIME@&ID=MyPhone
into the "Custom Web Server URL" setting field.  You
can replace MyPhone with any text that you want to
use to identify the individual phone or other
device.

Written in HTML, CSS, PHP, and JavaScript.  Also
uses the Google Maps JavaScript API and requires
MySQL.

One final note: I am no programmer.  This started as 
just a fun weekend project.  As such, it may contain
inefficiencies and has no security features
whatsoever.  Use at your own risk.

For questions, e-mail kepesk at gmail dot com

If you like this script and wish to do something in
return, please consider donating to my fundraiser to
get my brother some needed surgery at:
http://www.FixMyBrothersButt.com/

******************************************************

Release Notes

V. 3.0
* Added the ability to track multiple devices in the
  same database, differentiated by the name field.
  On opening the page the first listed device will be
  displayed, and a new dropdown list has been added
  to choose other devices.
* Added the defaultName user-specified variable to set
  the device that is displayed by default.
* Added support for Self-Hosted GPS Tracker for
  Android, with the limitation that all coordinates
  from this app will be listed as "Self-Hosted GPS
  Tracker" and accuracy is assumed to be 100 meters.
* Added support for Open GPS Tracker for Android.

V. 2.9
* Added XML POST request support.  Specifically, the
  Btraced protocol is now supported, but only some
  recieved data is actually used at this time.
  Support for all data passed by the Btraced protocol
  is planned for a future version.

V. 2.8
* Fixed a problem processing addresses with
  apostrophes or quotations.

V. 2.7
* Major rewrite of the address consolidation code.
  The first stage now consolidates identical addresses
  and addresses with the same number on two adjoining
  streets, and the second sorts by a partially-
  reversed address string to identify locations that
  are both close enough together and have low enough
  accuracy to determine a high probability that they
  were recorded at the same address.  This increased
  the accuracy of address identification, and the main
  sources of errors should now be a lack of sufficient
  coordinate data and flaws in Google's address
  database.
* Slightly reduced memory requirements.
* Corrected a text error that cropped up when there
  was an ampersand in an address.
* Corrected a consolidation error that occured when
  a stay consisted of two gathered coordinate sets
  with different accuracies.
* Slightly increased the default and recommended
  $errorCorrectCell values to correct for observed
  inaccuracies.
* Added a step to eliminate inaccurate coordinates
  that were likely derived via wifi by a stationary
  device.
* Corrected a bug in the address table that caused
  duplicate entries when the current address was not
  accurate enough to be included in a stay but was
  still valid.
* Corrected a bug that caused coordinates without
  addresses to be analyzed for the address table.
  This reduced the memory requirement and processing
  time of the script.

V. 2.6
* Corrected for a pattern where coordenates gathered
  through cell triangulation with an accuracy of
  exactly 100m were egregiously inaccurate.
* Lowered the default and recommended
  $errorCorrectCell values back to V. 2.1 levels,
  which gives greater accuracy once the above anomaly
  has been corrected.
* Added a multiplier equasion to the required
  reported accuracy for the elimination of a set of
  coordinates.  This mimics the former LiveUpdates
  adjustments in that it allows for the display of
  finer movements when the device is reporting
  coordinates frequently.
* Eliminated the $errorCorrectAddress variable and
  instead used the already-present accuracy data to
  determine consolidation of addresses.  This
  improves the accuracy of reverse-geolocation as
  well as the accuracy of determining which stays are
  at the same address.
* Fixed an error that caused nonstandard characters
  in addresses to be displayed incorrectly.
* Added a 'Display More Addresses' link to list six
  additional addresses when available.

V. 2.5
* Major rewrite of the code to improve the way that
  the Phone Finder groups information derived from a
  recieved coordinate set.  This will improve my
  ability to provide future improvements and fine-tune
  the script.
* Abandoned the Live Updates adjustments since
  mapping apps (including Device Locator) that use a
  change of cell towers to trigger a capture of
  coordinates would frequently and inappropriately
  trigger this functionality.  This may return in the
  future.
* The Phone Finder now accepts the undocumemted
  timediff variable from the Device Locator website.
  This means that the script will properly adjust the
  coordinates' time of recording based on how long
  they were stored before being transmitted.  Device
  Locator users should adjust their 'Forwarding URL'
  setting to match the instructions above.  Use of the
  timediff variable is optional, but strongly
  recommended.
* Improved the address consolidation algorithm to
  better consolidate locations that are actually at
  the same address, even if Google returns two
  different addresses for those locations.
* Fixed a bug that sometimes listed double entries for
  addresses in the address table.
* Adjusted the algorithm that determines how long the
  device has remained in the same location so that it
  does not include adjacent reported coordinates of
  questionable accuracy.  This will reduce the length
  but improve the accuracy of time stayed listings.
* Color-coded the map markers, address table, and
  coordinate table to highlight when the device was 
  stationary in the same location at different times.
* Improved the accuracy of the algorithm that
  determines whether a set of coordinates were
  gathered from cell tower locations, wifi
  triangulation, or GPS, mainly  by comparing the
  returned accuracy against standard GPS accuracies
  that are based on the number of GPS satellites
  visible to the device.
* Increased the default and recommended values for the
  $errorCorrectCell variable due to improved accuracy
  in the code that determines when this will be used.
* Implemented a second check to the address cache after
  addresses are consolidated to improve accuracy of
  listed addresses.
* Removed the shadow from the red marker to make it
  similar to the other markers.
* The coordinates table now displayes minutes for long
  stays.
* Added a user-defined $errorCorrectAddress veriable.
  This is similar to other error correct variables,
  except it specifies error tolerance when determining
  whether two positions are at the same address.
* Added accuracy circles around all of the secondary
  map markers.
* Adjusted all displayed accuracy information to take
  into account the errorCorrect variables.
* Added the $errorCorrectOther user-defined variable
  to add an accuracy multiplier to any coordinates
  that were either determined by wifi triangulation
  or where the source was indeterminable.  This
  dramatically increases the accuracy of reverse-
  geolocation.
* Added accuracies to address listings.
* Rounded all accuracy listings to two decimal places,
  since listing them to the millimeter seems excessive.


V. 2.1
* Improved the speed and memory usage of the code that
  reads the coordinate table.
* Improved the speed and memory usage of the code that
  reads the cache table.
* Decreased the amount of loading text printed while
  geocoding and broke it up with spaces.
* Improved (but did not quite perfect) consolidation
  of same-address coordinates for correction of Google
  reverse-geolocation errors.
* Added 2 weeks, 3 months, and 6 months time interval
  options.
* Updated instructions for use of this script with the
  Device Locator app.


V. 2.01
* A fix to a critical bug in 2.0 that prevented the
  red marker from being displayed when the current
  position wasn't accurate enough for address
  identification.  Fixing this bug also simplified the
  map drawing code somewhat.

V. 2.0
* Added address identification.  A new table between
  the map and the coordinate table displays the three
  street addresses where the phone has most frequently
  been and that can also be identified by the Google
  API.  It also displays the address associated with
  the most recent coordinates.
* Added a MySQL table to cache addresses and reduce
  calls to the Google geocoding service.  This table
  is automatically created on the first run and does
  not need to be created by the user.
* Added code to eliminate errors related to
  identification of certain addresses that are
  adjacent to street corners, as well as some "next
  door" errors due to inaccurate GPS coordinates.
* Added map markers for the street address
  identification.
* The map now takes a nontrivial amount of time to
  load, the length of which depends on the speed of
  the server, its internet connection, the amount of
  coordinates that the script is analyzing, and the
  amount of addresses that have been cached.  Added
  text displaying the status of the script as it is
  loading.
* Added safeguards to stop address identification
  queries when the daily query limit is reached.
  In this case, the script skips display of addresses
  and processes the rest of the data normally.
* Abandoned the markers for the second and third most
  frequently visited locations since those were
  obnoxious and not typically necessary.
* Modified the map to always display the marker for
  the most recently visited location in front of
  other markers.
* Fixed a minor error in the identification of
  coordinates that have been determined through cell
  tower triangulation.
* Added automatic table creation on the script's first
  run.  The user no longer has to manually create
  the MySQL coordinates table.
* Improved error messages for MySQL failures.
* Added an error message for a lack of coordinates.
* The 'Time to Analyze' form will now display the
  currently selected time interval if it is one of the
  choices in the form.
* Applied correct pluralization to all text in the
  script.
* Fixed a bug that caused the earliest analyzed
  location not to be displayed on the map.
* Corrected the map so it always includes the accuracy
  circle within its initial bounds.
* Fixed multiple errors and unpredictable behaviors
  related to displaying the map and tables when the
  database contains less than three sets of
  coordinates.
* Removed a piece of testing code that had been left
  in the last few versions.
* Fixed an error that prevented the 'Time to Analyze'
  form from working on some servers.

V. 1.5
* Improved the visibility of the LiveUpdate color
* Gaps in the data that exceed 12 hours will now be
  displayed regardless of whether or not the phone has
  moved, assuming the $interval variable is used.
* Fixed a bug that could cause problems with the MySQL
  database if the script has been running for a long
  time.
* Added map markers and table colors for the last
  three visited places.
* Fixed a table color display bug related to
  LiveUpdates.
* Added a table row color legend.
* Added "1 Year" and "All" choices to the "Time to
  Analyze" menu.
* Modified the LiveUpdates correction to also take
  into account the user-defined $errorCorrect
  accuracy multiplier variable.
* Minor interface enhancements.
* Slightly increased the accuracy of LiveUpdate
  detection

V. 1.4
* Added tests to recognize LiveUpdate coordinates and
  significantly reduced the stay threshhold and error
  correction when LiveUpdating. This reduces (but does
  not completely eliminate) unpredictable behavior
  when using this script with LiveUpdates.
* Marked LiveUpdate coordinates in yellow on the
  coordinate table.
* Fixed an error that made the map fail to display
  properly on Internet Explorer.
* Added code to make the lines thinner when a lot of
  them are being displayed.
* Made an adjustment to the cell tower algorithm to
  display certain good coordinates that were
  eliminated in version 1.3

V. 1.3
* Fixed a bug where the page displayed more locations
  than it should have when the phone hadn't
  consistently reported its locations to the server.
* Added an optional parameter for the Cerebro GPS
  interval.  If it is used, then this page will
  display gaps in the coordinates and specify the
  reasons for those gaps.
* Added an accuracy multiplier parameter to allow easy
  fine-tuning of which coordinates gathered by cell
  triangulation are considered stationary instead of
  the previous static 12km limit.  I debated doing
  this since it seems to make the algorithm more
  accurate in some places and less so in others, but
  overall it seems to allow slightly more accurate
  coordinates to be displayed than before.
* Eliminated the 30km upper accuracy limit on
  accepted coordinates since the above change seems
  to take care of this in all relevant cases.

V. 1.2
* Added true averaging of stationary coordinates for
  better error correction.
* Imoroved basic error correction to base it on the
  given coordinate accuracies instead of the previous
  static algorithm.  This eliminates some bad error
  correction decisions made when coordinates have been
  determined at locations with poor GPS signals.
* Fixed a bug that caused coordinates to be included
  in a consolidated stay listing when the true
  location of those coordinates were indeterminable.
* Simplified cell tower coordinate detection and fixed
  a bug that caused those coordinates to occasionally
  be improperly listed when better coordinates are
  available.
* Fixed two bugs that caused some stationary
  coordinates to be incorrectly interpeted as non-
  stationary.
* Added an accuracy multiplier parameter to allow easy
  fine-tuning of which coordinates are considered
  stationary.

V. 1.1
* Added calculations to base error correction on
  great-circle distances instead of latitude and
  longitude differences.  This improves the accuracy
  of error correction and eliminates unpredictable
  behavor at extremely high and low longitudes.
* Fixed a rare bug concerning altitude correction
  near sea level.
* Removed extraineous tests that were intended to help
  determine whether particular coordinates were
  found by geolocation using cell towers.
* Removed an error correction test that did not appear
  to affect the calculations in any way.
* Added new error correction test that can eliminate a
  particular type of error which may be caused by
  geolocation using wifi networks.
* Eliminated egregiously inaccurace addresses from the
  address table.

V. 1.0
* Initial Release

******************************************************

Known Issues

* The algorithm cannot tell the difference between
  coordinates that were gathered using wifi network
  triangulation and some sea-level coordinates.  This
  may rarely cause valid sea-level coordinates to be
  inappropriately discarded.
* The loading text does not display on all web
  browsers and server configurations, most notably
  Safari for iOS.
* Google has a limit on the number of server calls
  that can be used for address identification in a
  single day.  As of August 2010, this limit is 2,500.
  While this script takes measures to limit the number
  of such calls made, address identification will
  temporarily stop working if the script is reloaded
  a couple hundred times in a given day or if an
  excessive amount of coordinates are being processed.
  If this happens, the rest of the script will still
  function normally, but the address table and markers
  will dissappear until the next day.
* There is some kind of latency that sometimes causes
  the script to think geocoding calls have run out
  when they have not.  This is a fixable error, but
  I have not pinned it down yet

******************************************************

This program is free software: you can redistribute it
and/or modify it under the terms of the GNU General
Public License as published by the Free Software
Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public
License for more details.

For a copy of the GNU General Public License, see
<http://www.gnu.org/licenses/>. 

*****************************************************/

ini_set("memory_limit","256M");  //This gives the script enough memory to load the coordinates that would be gathered in a year of typical use.  Some web services will not allow this to be set so high, if at all.  If you get an error because of this, try adjusting it or commenting it out.

@apache_setenv('no-gzip', 1);
@ini_set('zlib.output_compression', 0);
@ini_set('implicit_flush', 1);
for ($i = 0; $i < ob_get_level(); $i++) { ob_end_flush(); }
ob_implicit_flush(1);

//***************User-defined variables start here

$user="name";  // Enter your database username here
$password="password";  // Enter your database password here
$database="tracking";  // Enter your database name here
$errorCorrect=1.1;  // Accuracy multiplier for use in correcting errors when the phone appears stationary.  A higher number interpets more coordinates as stationary, and a lower number displays more fine movements.  Values between 0.75 and 3 seem to work best.  Default is 1.1.
$errorCorrectCell=32;  // Same as above, except it only applies to coordinates obtained using cell tower triangulation; the accuracies on these tend to be vastly overstated.  Change this if you are getting displayed coordinates way off of your actual location, or if coordinates gathered while traveling are not displaying.  I recommend 10-30 for accuracy in urban areas, and 20-40 for rural areas.  Default is 32.
$errorCorrectOther=3; // Same as above, except it applies to coordinates that cannot be identified firmly as gathered from GPS or cell tower triangulation.  This category includes wifi triangulation.  I recommend sticking with the default of 3 under most circumstances, but I've put it here in case you want to tinker with it.
$interval="1800";  // Optional parameter.  This is the frequency in seconds that your location is updated (not counting cell tower changes).  Identical to the GPS interval setting in your reporting software.  Entering this will allow the page to accurately display gaps in the data.
$defaultName="";  // Optional parameter.  This is the name of the default device to be displayed on the map page.  If nothing is entered, the earliest listed device will be shown by default.

//***************End of user-defined variables

// Get great circle distances between coordinates in meters.  For use in error correction.
function getDistance($lat1, $long1, $lat2, $long2){
	$distance = (3958*3.1415926*sqrt(($lat2-$lat1)*($lat2-$lat1) + cos($lat2/57.29578)*cos($lat1/57.29578)*($long2-$long1)*($long2-$long1))/180)*1000;
	return $distance;
}

	// Reverse a string while leaving numbers in their previous order
	function utf8_strrev($str, $reverse_numbers = true){
//print($str.' -> ');
		$pattern = $reverse_numbers ? '/./us' : '/(\d+)?./us';
		preg_match_all($pattern, $str, $ar);
//print(join('',array_reverse($ar[0])).'<br>
//');
//flush();
		return join(' ',array_reverse($ar[0]));
	}

$recordIndexCurrent = 0;

class position {

	protected $lat = -1;
	protected $long = -1;
	protected $acc = -1;
	protected $alt = -1;
	protected $altAcc = -1;
	protected $batt = -1;
	protected $nextPosition;
	protected $prevPosition;
	protected $deviceID;
	protected $address;

public $eliminate = 0;

	public function getAcc(){ 
		return $this->acc;
	}

	public function getNextPosition(){ 
		return $this->nextPosition;
	}

	public function getPrevPosition(){ 
		return $this->prevPosition;
	}

	public function getLat(){ 
		return $this->lat;
	}

	public function getLong(){ 
		return $this->long;
	}

	public function getAlt(){ 
		return $this->alt;
	}

	public function getAddress(){ 
		return $this->address;
	}

	public function getTimeTotal(){
		return 0;
	}

	public function getDistanceSincePrevPosition(){ //returns in meters

		if(!isset($this->prevPosition)){
			return -1;
		}
		else{
			return getDistance($this->lat, $this->long, $this->prevPosition->getLat(), $this->prevPosition->getLong());
		}
	}

	public function getLatestTime(){ 
		return -1;
	}

	public function getEarliestTime(){ 
		return -1;
	}

	public function getLatestRecordIndex(){ 
		return -1;
	}

	public function getEarliestRecordIndex(){ 
		return -1;
	}

	public function getLowestBatt(){
		return -1;
	}

	public function getHighestBatt(){
		return -1;
	}

	public function getSpeedSincePrev(){ //return in kph
		return -1;
	}

	public function setAcc($var){
		$this->acc = $var;
	}

	public function setAlt($var){
		$this->alt = $var;
	}

	public function setAddress($var){
		$this->address = $var;
	}

	public function setNextPosition($var){
		$this->nextPosition = $var;
	}

	public function setPrevPosition($var){
		$this->prevPosition = $var;
	}

	public function getMethod(){ // 0=Cell tower triangulation 1=Wifi triangulation or better 2=GPS
		return -1;
	}

	public function getNumContainedCoords(){
		return 1;
	}
	public function isLastPosition(){
		return !isset($this->nextPosition);
	}

}

class Coordinates extends position {

	private $time;
	private $prev;
	private $next;
	private $stay = null;
	private $recordIndex;

	function __construct($nlat, $nlong, $nalt, $nacc, $ntime, $nbatt, $ndeviceID, $recordIndexCurrent){
		$this->lat = $nlat;
		$this->long = $nlong;
		$this->alt = $nalt;
		$this->acc = $nacc;
		$this->time = $ntime;
		$this->batt = $nbatt;
		$this->deviceID = $ndeviceID;

		$this->recordIndex = $recordIndexCurrent;
	}

	public function getNext(){
		return $this->next;
	}

	public function getPrev(){
		return $this->prev;
	}

	public function getStay(){
		return $this->stay;
	}

	public function isLast(){
		return !isset($this->next);
	}

	public function getLatestTime(){
		return $this->time;
	}

	public function getEarliestTime(){
		return $this->time;
	}

	public function getLatestRecordIndex(){
		return $this->recordIndex;
	}

	public function getEarliestRecordIndex(){
		return $this->recordIndex;
	}

	public function getLowestBatt(){
		return $this->batt;
	}

	public function getHighestBatt(){
		return $this->batt;
	}

	public function getTimeSincePrev(){

		if(!isset($this->prev)){
			return -1;
		}
		else{
			return abs(strtotime($this->time) - strtotime($this->prev->getLatestTime()));
		}
	}

	public function getSpeedSincePrev(){

		if(!isset($this->prev)){
			return -1;
		}
		else{
			return (($this->getDistanceSincePrev() / 1000) / ($this->getTimeSincePrev() / 60));
		}
	}

	public function getDistanceSincePrev(){ //returns in meters
		if(!isset($this->prev)){
			return -1;
		}
		else{

			return getDistance($this->lat, $this->long, $this->prev->getLat(), $this->prev->getLong());

		}
	}

	public function getMethod(){
		if($this->acc > -1 && $this->alt > -1){
			//Check for known accuracies that identify GPS-determined coordinates
			if((round($this->acc, 2) == 47.42) || (round($this->acc, 2) == 17.07) || (round($this->acc, 2) == 76.36) || (round($this->acc, 2) == 162.96) || (round($this->acc, 2) == 9.11)){
				return 2;
			}
			//If no altitude is reported and the accuracy is less than 100, then this is likely at least wifi triangulation.
			if(($this->alt <= 0) && ($this->acc < 100)){
				return 1;
			}
			//Otherwise, If no altitude is reported and the accuracy is an integer, this is almost certainly a single cell tower location.
			if(($this->alt <= 0) && (floor($this->acc) == $this->acc)){
				return 0;
			}
			//If no altitude is reported and the accuracy is above 500, this is likely cell triangulation.
			if(($this->alt <= 0) && ($this->acc >= 500)){
				return 0;
			}
			//If the accuracy is above 1000, this is almost certainly cell triangulation
			if($this->acc >= 1000){
				return 0;
			}
			return 1;
		}
		else{
			return -1;
		}
	}

	public function getAcc(){
		global $errorCorrectCell;
		global $errorCorrectOther;
		global $errorCorrect;
		if($this->getMethod() == 0){ //cell

			//Correcting for a pattern where cell towers that give an accuracy of exactly 100 are particularly inaccurate.
			if($this->acc == 100){
				return (100 * $errorCorrectCell * 1.77);
			}

			return ($this->acc * $errorCorrectCell);
		}
		if($this->getMethod() == 1){ //cell
			return ($this->acc * $errorCorrectOther);
		}
		return ($this->acc * $errorCorrect);
	}

	public function setAcc($var){
		$this->acc = $var;
	}

	public function setStay($var){
		$this->stay = $var;
	}

	public function setNext($var){
		$this->next = $var;
	}

	public function setPrev($var){
		$this->prev = $var;
	}

	public function isStay(){
		if(isset($this->stay)){
			return true;
		}
		return false;
	}

}

class Stay extends position {

	private $coords = array();
	private $lowestBatt;
	private $highestBatt;
	private $method;

	function __construct($coord1, $coord2){
		$this->coords[0] = $coord1;
		$this->coords[1] = $coord2;
		$this->timeStart = $coord1->getLatestTime();
		$this->timeEnd = $coord2->getEarliestTime();
		if($coord1->getAcc() > $coord2->getAcc()){
			$this->acc = $coord2->getAcc();
			$this->lat = $coord2->getLat();
			$this->long = $coord2->getLong();
			$this->alt = $coord2->getAlt();
		}
		else if($coord2->getAcc() > $coord1->getAcc()){
			$this->acc = $coord1->getAcc();
			$this->lat = $coord1->getLat();
			$this->long = $coord1->getLong();
			$this->alt = $coord1->getAlt();
		}
		else{
			$this->acc = $coord1->getAcc();
			$this->lat = ($coord1->getLat() + $coord2->getLat()) / 2;
			$this->long = ($coord1->getLong() + $coord2->getLong()) / 2;
			if($coord1->getAlt() == 0 && $coord2->getAlt() != 0){
				$this->alt = $coord2->getAlt();
			}
			elseif($coord1->getAlt() != 0 && $coord2->getAlt() == 0){
				$this->alt = $coord1->getAlt();
			}
			else{
				$this->alt = ($coord1->getAlt() + $coord2->getAlt()) / 2;
			}
		}
		//Set Battery
		if($coord1->getLowestBatt() > $coord2->getLowestBatt()){
			$this->lowestBatt = $coord2->getLowestBatt();
			$this->highestBatt = $coord1->getLowestBatt();
		}
		else{
			$this->lowestBatt = $coord1->getLowestBatt();
			$this->highestBatt = $coord2->getLowestBatt();
		}
		//Set Method
		if($coord1->getMethod() > $coord2->getMethod()){
			$this->method = $coord1->getMethod();
		}
		else{
			$this->method = $coord2->getMethod();
		}
	}

	public function getLowestBatt(){
		return $this->lowestBatt;
	}

	public function getHighestBatt(){
		return $this->highestBatt;
	}

	public function getMethod(){
		return $this->method;
	}

	public function getEarliestTime(){
		return $this->coords[count($this->coords)-1]->getEarliestTime();
	}

	public function getLatestTime(){
		return $this->coords[0]->getLatestTime();
	}

	public function getTimeTotal(){ //returns in seconds
		return abs(strtotime($this->getLatestTime()) - strtotime($this->getEarliestTime()));
	}

	public function getEarliestRecordIndex(){
		return $this->coords[count($this->coords)-1]->getLatestRecordIndex();
	}

	public function getLatestRecordIndex(){
		return $this->coords[0]->getEarliestRecordIndex();
	}

	public function getNumContainedCoords(){
		return count($this->coords);
	}

	public function addCoords($var){
		$this->coords[] = $var;
		if($var->getAcc() < $this->acc){
			$this->acc = $var->getAcc();
			$this->lat = $var->getLat();
			$this->long = $var->getLong();
			$this->alt = $var->getAlt();
		}
		elseif($var->getAcc() == $this->acc){
			$i = 0;
			$j = 0;
			$totalLat = 0;
			$totalLong = 0;
			$totalAlt = 0;
			while(isset($this->coords[$i])){
				if($this->coords[$i]->getAcc() == $this->acc){
					$j++;
					$totalLat += $this->coords[$i]->getLat();
					$totalLong += $this->coords[$i]->getLong();
					$totalAlt += $this->coords[$i]->getAlt();
				}
			$i++;
			}
			$this->lat = $totalLat / $j;
			$this->long = $totalLong / $j;
			$this->alt = $totalAlt / $j;
		}
		//Set Battery
		if($var->getLowestBatt() < $this->lowestBatt){
			$this->lowestBatt = $var->getLowestBatt();
		}
		elseif($var->getHighestBatt() > $this->highestBatt){
			$this->highestBatt = $var->getHighestBatt();
		}
		//Set Method
		if($var->getMethod() > $this->method){
			$this->method = $var->getMethod();
		}
		
		//usort($this->coords, "sortCoordsByTime");
	}

	public function setAddress($var){
		$this->address = $var;
		$i = 0;
		while(isset($this->coords[$i])){
			$this->coords[$i]->setAddress($var);
			$i++;
		}
	}

	public function getAddress(){ 
		return $this->address;
	}

}

class Address extends Position {

	private $positions = array();
	private $addressString;
	private $sortString;
	private $timeTotal = 0;

	function __construct($position){
		$this->positions[0] = $position;
		$this->lat = $position->getLat();
		$this->long = $position->getLong();
		$this->acc = $position->getAcc();
		$this->alt = $position->getAlt();
		$this->timeTotal += $position->getTimeTotal();
	}

	function addPosition($position){
		$this->positions[] = $position;
		$this->timeTotal += $position->getTimeTotal();
		if($position->getAcc() < $this->acc){
			$this->acc = $position->getAcc();
			$this->lat = $position->getLat();
			$this->long = $position->getLong();
			$this->alt = $position->getAlt();
		}
		elseif($position->getAcc() == $this->acc){
			$i = 0;
			$j = 0;
			$totalLat = 0;
			$totalLong = 0;
			$totalAlt = 0;
			while(isset($this->positions[$i])){
				if($this->positions[$i]->getAcc() == $this->acc){
					$j++;
					$totalLat += $this->positions[$i]->getLat();
					$totalLong += $this->positions[$i]->getLong();
					$totalAlt += $this->positions[$i]->getAlt();
				}
			$i++;
			}
			$this->lat = $totalLat / $j;
			$this->long = $totalLong / $j;
			$this->alt = $totalAlt / $j;
		}
	}

	function addPositions($nPositions){

		$k = 0;
		while(isset($nPositions[$k])){
			$position = $nPositions[$k];
			$this->positions[] = $position;
			$this->timeTotal += $position->getTimeTotal();
			if($position->getAcc() < $this->acc){
				$this->acc = $position->getAcc();
				$this->lat = $position->getLat();
				$this->long = $position->getLong();
				$this->alt = $position->getAlt();
			}
			elseif($position->getAcc() == $this->acc){
				$i = 0;
				$j = 0;
				$totalLat = 0;
				$totalLong = 0;
				$totalAlt = 0;
				while(isset($this->positions[$i])){
					if($this->positions[$i]->getAcc() == $this->acc){
						$j++;
						$totalLat += $this->positions[$i]->getLat();
						$totalLong += $this->positions[$i]->getLong();
						$totalAlt += $this->positions[$i]->getAlt();
					}
					$i++;
				}
				$this->lat = $totalLat / $j;
				$this->long = $totalLong / $j;
				$this->alt = $totalAlt / $j;
			}
			$k++;
		}
	}

	public function getTimeTotal(){
		return $this->timeTotal;
	}

	public function getPositions(){
		return $this->positions;
	}

	public function getAddressString(){
		return $this->addressString;
	}

	public function getSortString(){
		return $this->sortString;
	}

	public function setAddressString($var){
		$this->addressString = $var;
		$this->sortString = trim(utf8_strrev(strtolower($var), false));
	}

}

function addressCompareByCoords($a, $b){
	if($a->getLat() != $b->getLat()){
		return floor(($a->getLat() + $b->getLat()) * 1000);
	}
	return floor(($a->getLong() + $b->getLong()) * 1000);
}


mysql_connect('localhost',$user,$password);
mysql_select_db($database) or die( "Unable to select database: ".mysql_error());

$query = "CREATE TABLE IF NOT EXISTS `".$database."`.`coordinates` (
`latitude` double NOT NULL ,
`longitude` double NOT NULL ,
`accuracy` double NOT NULL ,
`altitude` double NOT NULL ,
`name` varchar( 40 ) NOT NULL ,
`time` datetime NOT NULL ,
`battery` double NOT NULL ,
PRIMARY KEY ( `time` ) );";
$result = mysql_query($query) or die('Error Creating Coordinate Table:' . mysql_error());

$query = "CREATE TABLE IF NOT EXISTS `".$database."`.`addressCache` (
`coordinates` VARCHAR( 19 ) NOT NULL ,
`address` TEXT NOT NULL ,
`time` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
PRIMARY KEY ( `coordinates` )
) ENGINE = MYISAM ;";
$result = mysql_query($query) or die('Error Creating Cache Table:' . mysql_error());

//Truncate coordinates to 5 decimal places for caching purposes.
function coordinateTruncate($lat, $long){
	return (substr($lat, 0, (strpos($lat, '.') + 6)).','.substr($long, 0, (strpos($long, '.') + 6)));
}

//Get s if plural
function plu($num){
	if($num != 1){
		return 's';
	}
	return;
}

if($_GET['steps'] != ''){
	$amnt = $_GET['steps'];
}
else{
	$amnt = 72;
}

//If coordinates are being automatically added
if($_GET['LATITUDE'] != '' && $_GET['APP'] == ''){
	if($_GET['TIMEDIFF'] != ''){
	$query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$_GET['LATITUDE']."','".$_GET['LONGITUDE']."','".$_GET['ALTITUDE']."','".$_GET['ACCURACY']."','".$_GET['ID']."','".date("Y-m-d H:i:s",(time()+$_GET['TIMEDIFF']))."','".$_GET['BATTERYLEVEL']."');"; 
	}
	else{
	$query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$_GET['LATITUDE']."','".$_GET['LONGITUDE']."','".$_GET['ALTITUDE']."','".$_GET['ACCURACY']."','".$_GET['ID']."',NOW(),'".$_GET['BATTERYLEVEL']."');"; 
	}
	$result = mysql_query($query) or die('mysql error:' . mysql_error());
}

//If coordinates are being automatically added by Self-Hosted GPS Tracker
elseif($_GET['lat'] != ''){
        $query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$_GET['lat']."','".$_GET['lon']."','0','100','Self-Hosted GPS Tracker','0','0');";
        $result = mysql_query($query) or die('mysql error:' . mysql_error());
}

//If coordinates are being automatically added by Open GPS Tracker
elseif($_GET['APP'] == 'OGT'){
        $query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$_GET['LATITUDE']."','".$_GET['LONGITUDE']."','0','".$_GET['ACCURACY']."','".$_GET['ID']."','".date("Y-m-d H:i:s",($_GET['TIME'] / 1000))."','0');";
        $result = mysql_query($query) or die('mysql error:' . mysql_error());
}

//?APP=OGT&LONGITUDE=@LON@&LATITUDE=@LAT@&ACCURACY=@ACC@&SPEED=@SPEED@&BEARING=@BEAR@&TIME=@TIME@&ID=MyPh

//If coordinates are being manually added
elseif($_GET['LATITUDE'] != '' && $_GET['manual'] == 'yes'){
	$query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$_GET['LATITUDE']."','".$_GET['LONGITUDE']."','0','100','".$_GET['ID']."','".$_GET['TIME']."','-1');"; 
	$result = mysql_query($query) or die('mysql error:' . mysql_error());
}

//If XML coordinates are being automatically added
elseif (file_get_contents('php://input') != null) {

	// Get the received data from the device (XML data)
	$xml_data = @file_get_contents('php://input');
	// Try to load the XML
	$xml = simplexml_load_string($xml_data);

	// If there was an error report it...
	if ($xml == false) {
		// Error loading XML..., send it back to the device
		echo '{ "id":902, "error":true, "message":"Cant load XML", "valid":true }';
	}
	else {
   
		// Get device identification
		$deviceId = $xml->devId;

		$timeOffset = $xml->timeOffset;
   
		// Prepare list of points
		$goodPointsList = "";
      
		// Start processing each travel
		foreach ($xml->travel as $travel) {
      
			// Get travel common information
			$travelId = $travel->id;
			$travelName = $travel->description;
			$travelLength = $travel->length;
			$travelTime = $travel->time;
			$travelTPoints = $travel->tpoints;
      
			// Prepare the succesful points
			$goodPointsList = '';
      
			// Process each point
			foreach ($travel->point as $point) {
         
				// Get all the information for this point
				$pointId = $point->id;
				$pointDate = $point->date;
				$pointLat = $point->lat;
				$pointLon = $point->lon;
				$pointSpeed = $point->speed;
				$pointCourse = $point->course;
				$pointHAccu = $point->haccu;
				$pointBatt = $point->bat;
				$pointVAccu = $point->vaccu;
				$pointAltitude = $point->altitude;
				$pointContinous = $point->continous;
				$pointTDist = $point->tdist;
				$pointRDist = $point->rdist;
				$pointTTime = $point->ttime;
         
				// Create SQL sentence
				$query = "insert INTO coordinates (latitude,longitude,altitude,accuracy,name,time,battery) VALUES ('".$pointLat."','".$pointLon."','".$pointAltitude."','".$pointHAccu."','".$deviceId."','".date("Y-m-d H:i:s",(round($pointDate)-$timeOffset))."','".$pointBatt."') ON DUPLICATE KEY UPDATE latitude = latitude"; 
	
				$result = mysql_query($query) or die('mysql error:' . mysql_error());

				// Check if the Insert was succesful
				if ($result != false) {
					// Add this point to the saved points
					$goodPointsList .= $pointId.",";
				}
				else {
					echo "SQL Error: ".mysql_error();   
			        }

			}   
		}

		// Check if there was points
		if ($goodPointsList != "") {
			// Remove last comma
			$goodPointsList = substr($goodPointsList, 0, -1);


			// Send back the answer for the saved points
			echo '{"id":0, "tripid":'.$travelId.',"points":['.$goodPointsList.'],"valid":true}';
			return;
   		}
		else {
			// Just OK, the code should never reach here as we always have points
			echo '{"id":0, "tripid":'.$travelId.',"valid":true}';  
			return;    
   		}

	}
}

else{

function datetime_diff($d1, $d2){
    $d1 = (is_string($d1) ? strtotime($d1) : $d1);
    $d2 = (is_string($d2) ? strtotime($d2) : $d2);

    $diff_secs = abs($d1 - $d2);
    $base_year = min(date("Y", $d1), date("Y", $d2));

    $diff = mktime(0, 0, $diff_secs, 1, 1, $base_year);
    return array(
        "years" => date("Y", $diff) - $base_year,
        "months_total" => (date("Y", $diff) - $base_year) * 12 + date("n", $diff) - 1,
        "months" => date("n", $diff) - 1,
        "days_total" => floor($diff_secs / (3600 * 24)),
        "days" => date("j", $diff) - 1,
        "hours_total" => floor($diff_secs / 3600),
        "hours" => date("G", $diff),
        "minutes_total" => floor($diff_secs / 60),
        "minutes" => (int) date("i", $diff),
        "seconds_total" => $diff_secs,
        "seconds" => (int) date("s", $diff)
    );
}


//Grab the most recent data

	$name = '';
	if($_GET['name']){
		$name = $_GET['name'];
	}
	elseif($defaultName){
		$name = $defaultName;
	}
	else{
                $query="SELECT DISTINCT(name) FROM coordinates";

                $result = mysql_query($query) or die('mysql error:' . mysql_error());
                $row = mysql_fetch_row($result);
                $name .= $row[0];
	}

	if($amnt < 0){
		$query = "select * FROM coordinates WHERE name = '".$name."' ORDER BY time DESC;";
	}
	else{
		$query = "select * FROM coordinates WHERE (name = '".$name."' AND time >= DATE_SUB(NOW(), INTERVAL ".$amnt." Hour)) ORDER BY time DESC;";
	}
	$result = mysql_query($query) or die('Error Loading Coordinates:' . mysql_error());

	//Return the correct form for choosing the time interval.
	function intervalField($amnt){
		$output = ('Time to analyze: <select name="steps"><option');
		if($amnt == 6){
			$output .= (' selected');
		}
		$output .= (' value="6">6 hours</option><option');
		if($amnt == 12){
			$output .= (' selected');
		}
		$output .= (' value="12">12 hours</option><option');
		if($amnt == 24){
			$output .= (' selected');
		}
		$output .= (' value="24">1 day</option><option');
		if($amnt == 72){
			$output .= (' selected');
		}
		$output .= (' value="72">3 days</option><option');
		if($amnt == 168){
			$output .= (' selected');
		}
		$output .= (' value="168">1 week</option><option');
		if($amnt == 336){
			$output .= (' selected');
		}
		$output .= (' value="336">2 weeks</option><option');
		if($amnt == 720){
			$output .= (' selected');
		}
		$output .= (' value="720">1 month</option><option');
		if($amnt == 2192){
			$output .= (' selected');
		}
		$output .= (' value="2192">3 months</option><option');
		if($amnt == 4383){
			$output .= (' selected');
		}
		$output .= (' value="4383">6 months</option><option');
		if($amnt == 8766){
			$output .= (' selected');
		}
		$output .= (' value="8766">1 year</option><option');
		if($amnt == -1){
			$output .= (' selected');
		}
		$output .= (' value="-1">All</option></select>');
		return $output;
	}

	function nameField(){
		$output=' Device: <select name="name">';
		$query="SELECT DISTINCT(name) FROM coordinates";
        	$result = mysql_query($query) or die('mysql error:' . mysql_error());
                $row = mysql_fetch_row($result);
		while($row){
			$output .= '<option ';
			if($row[0] == $_GET['name']){
				$output .= 'selected="selected" ';
			}
			$output .= 'value="'.$row[0].'">'.$row[0].'</option>';
			$row = mysql_fetch_row($result);
		}
		$output .= "</select>";
		return $output;
	}

	if(mysql_num_rows($result) < 1){
		print('<html><head><title>Phone Finder</title></head><body>No coordinates found in the selected time interval.<br><br>If this is your first time running this script, please wait for the associated device to report its first set of coordinates.<br><br>If coordinates have been reported in the past, try selecting a larger time interval.<br><br><center><form action="" method="GET">');
		print(intervalField($amnt));
		print(nameField());
		print('<input type="submit" value="OK"></form></center></body></html>');
	}
	else{

//***************Build the coordinates list

	$row = mysql_fetch_row($result);

	$coordFirst = new coordinates($row[0],$row[1],$row[3],$row[2],$row[5],($row[6]*100),$row[4], $recordIndexCurrent);
	$recordIndexCurrent++;

	$coordCurr = $coordFirst;

	$i = 1;
	
	while ($i < mysql_num_rows($result)){
		$row = mysql_fetch_row($result);

		$coordCurr->setNext(new coordinates($row[0],$row[1],$row[3],$row[2],$row[5],($row[6]*100),$row[4], $recordIndexCurrent));
		$recordIndexCurrent++;
		$coordCurr->setNextPosition($coordCurr->getNext());
		$coordPrev = $coordCurr;
		$coordCurr = $coordCurr->getNext();
		$coordCurr->setPrev($coordPrev);
		$coordCurr->setPrevPosition($coordPrev);

		$i++;
	}

	unset($result);

//***************Start Printing the Page

	print('<html><head><title>Phone Finder</title>
	<style type="text/css">
		td {border-bottom-width:1px;border-bottom-color:#909090;border-bottom-style:solid;padding:3px;}
	</style>
	<meta name="viewport" content="width=480; maximum-scale=0.5; minimum-scale=0.333333333 user-scalable=no">
	<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=true"></script>

	<script type="text/javascript" language="JavaScript"><!--
		function HideContent(d) {
			if(d.length < 1) { return; }
			document.getElementById(d).style.display = "none";
		}
		function ShowContent(d) {
			if(d.length < 1) { return; }
			document.getElementById(d).style.display = "block";
		}
		function ReverseContentDisplay(d) {
			if(d.length < 1) { return; }
			if(document.getElementById(d).style.display == "none") { document.getElementById(d).style.display = "block"; }
			else { document.getElementById(d).style.display = "none"; }
		}
//--></script>


	</head><body style="border-width:0px; margin:0px; padding:0px;" onload="initialize(); setTimeout(function() { window.scrollTo(0, 1) }, 100);">');



//***************Build the stay list

	$coordCurr = $coordFirst;
	$coordPrev = null;
	$coordNext = $coordCurr->getNext();
	$stayFirst = null;
	$stayCurr = null;

	do{
		if(isset($coordPrev) && !($coordCurr->isLast())){

			$coordNext = $coordCurr->getNext();


			/*
			//Determine if the current coordinate should be eliminated entirely before the stay is built

			//If the current coordinate is close enough to the last one
			if(($coordCurr->getDistanceSincePrev() < $coordCurr->getAcc())
			//And the current coordinate is close enough to the next one
			  && ($coordNext->getDistanceSincePrev() < $coordCurr->getAcc())
			//And the current coordinate is inaccurate
			  && ($coordCurr->getAcc() > 4000)
			//And all three coordinates were submitted next to each other
			  && ($coordCurr->getTimeSincePrev() < ($interval * 1.1) && $coordNext->getTimeSincePrev() < ($interval * 1.1))){

				$coordPrev->setNextPosition($coordNext);

				if($coordPrev->isStay()){
					$coordPrev->getStay()->setNextPosition($coordNext);
					$coordNext->setPrevPosition($coordPrev->getStay());
				}
				else{
					$coordNext->setPrevPosition($coordPrev);
				}

				if(!($coordCurr->isLast())){
					$coordPrev = $coordCurr;
					$coordCurr = $coordNext;
				}

				continue;

			}*/


			//Determine if this and previous coordinates are part of the same stay

			//If the current coordinate is close enough to the last one
			if(($coordCurr->getDistanceSincePrev() < $coordCurr->getAcc())
			//And the last coordinate is close enough to this one
			  && ($coordCurr->getDistanceSincePrev() < $coordPrev->getAcc())
			//And if it has been fewer than twelve hours since the last coordinate was reported
			  && ($coordCurr->getTimeSincePrev() < 43200)){

				//If last position was a stay
				if($coordPrev->isStay()){

					//Add this coordinate to that stay
					$coordPrev->getStay()->addCoords($coordCurr);
					$coordCurr->setStay($coordPrev->getStay());
					if(!($coordCurr->isLast())){
						$coordPrev->getStay()->setNextPosition($coordNext);
						$coordNext->setPrevPosition($coordPrev->getStay());
					}
					else{
						$coordPrev->setNextPosition(null);
					}

				}
				//Else, last position was a coordinate, so make it a stay
				else{
					//Create a new stay
					$newStay = new Stay($coordPrev, $coordCurr);
					$stayCurr = $newStay;
					$coordPrev->setStay($newStay);
					$coordCurr->setStay($newStay);
					$coordPrevPrev = $coordPrev->getPrev();

					if(isset($coordPrevPrev)){
						if(!($coordPrevPrev->isStay())){
							$coordPrevPrev->setNextPosition($newStay);
						}
						else{
							$coordPrevPrev->getStay()->setNextPosition($newStay);
						}
					}
					if(!($coordCurr->isLast())){
						$coordNext->setPrevPosition($newStay);
						$newStay->setNextPosition($coordNext);
					}

				}

			}
			else{
				$stayCurr = null;
			}

		}
		if(!($coordCurr->isLast())){
			$coordPrev = $coordCurr;
			$coordCurr = $coordNext;
		}
	} while(!($coordCurr->isLast()));

	if($coordFirst->isStay()){
		$positionFirst = $coordFirst->getStay();
	}
	else{
		$positionFirst = $coordFirst;
	}

//***************Remove certain coordinates associated with a device that is not traveling where better coordinates are available

	$posCurr = $positionFirst;
	$posPrev = null;
	$posNext = $posCurr->getNextPosition();

	function getAccuracyMultiplier($deltaTime){
		/* Accuracy multiplier is defined as follows:
		   1+(0.77/(1+e^(($deltaTime-($interval+750)/2)/(($interval-750)/5))))
		   where $deltaTime is the time difference between two coordinate sets,
		   1.77 is the largest multiplier that is likely to retain accuracy,
		   and 750 seconds is considered a low enough coordinate interval to use that accuracy.
		*/
		return 1+(0.77/(1+exp(($deltaTime-($interval+750)/2)/(($interval-750)/5))));
	}

	if(isset($posNext)){
		do{
			$posPrev = $posNext->getPrevPosition();
			if(!isset($posPrev)){
				$posPrev = $posCurr;
			}
			$posCurr = $posCurr->getNextPosition();
			$posNext = $posCurr->getNextPosition();

			if(isset($posNext)){
				//If the current coordinate is a cell tower
				if(($posCurr->getMethod() == 0)

				    //And this coordinate is close enough to the last one, adjusted for the time between when they were gathered
				    && ((getDistance($posCurr->getLat(), $posCurr->getLong(), $posPrev->getLat(), $posPrev->getLong()) < (($posCurr->getAcc())*getAccuracyMultiplier($posCurr->getLatestTime()-$posPrev->getLatestTime())))
				    //And this coordinate is close enough to the next one, adjusted for the time between when they were gathered
				    && (getDistance($posCurr->getLat(), $posCurr->getLong(), $posNext->getLat(), $posNext->getLong()) < (($posCurr->getAcc())*getAccuracyMultiplier($posNext->getLatestTime()-$posCurr->getLatestTime()))))){

					//Eliminate this coordinate
					$posPrev->setNextPosition($posNext);
					$posNext->setPrevPosition($posPrev);

				}
				//Else if the current coordinate is from wifi
				elseif(($posCurr->getMethod() == 1)

				    //And this coordinate is close enough to the last one, adjusted for the time between when they were gathered
				    && ((getDistance($posCurr->getLat(), $posCurr->getLong(), $posPrev->getLat(), $posPrev->getLong()) < (0.44*($posCurr->getAcc())*getAccuracyMultiplier($posCurr->getLatestTime()-$posPrev->getLatestTime())))
				    //And this coordinate is close enough to the next one, adjusted for the time between when they were gathered
				    && (getDistance($posCurr->getLat(), $posCurr->getLong(), $posNext->getLat(), $posNext->getLong()) < (0.44*($posCurr->getAcc())*getAccuracyMultiplier($posNext->getLatestTime()-$posCurr->getLatestTime()))))){

					//Eliminate this coordinate
					$posPrev->setNextPosition($posNext);
					$posNext->setPrevPosition($posPrev);

				}
			}

		} while(!($posCurr->isLastPosition()));
	}


//***************Build the map
//Find the path's map center and boundaries and count the final position list

	$posCurr = $positionFirst;
	$positionCount = 1;

//print($posCurr->getLat());
	$maxLat = $posCurr->getLat();
	$maxLong = $posCurr->getLong();
	$minLat = $posCurr->getLat();
	$minLong = $posCurr->getLong();

	if(!$posCurr->isLastPosition()){
		do{

			$posCurr = $posCurr->getNextPosition();
			if($posCurr->getLat() > $maxLat){
				$maxLat = $posCurr->getLat();
			}
			elseif($posCurr->getLat() < $minLat){
				$minLat = $posCurr->getLat();
			}
			if($posCurr->getLong() > $maxLong){
				$maxLong = $posCurr->getLong();
			}
			elseif($posCurr->getLong() < $minLong){
				$minLong = $posCurr->getLong();
			}
			$positionCount++;

		} while(!($posCurr->isLastPosition()));
	}

	$centerLat = ($maxLat + $minLat) / 2;
	$centerLong = ($maxLong + $minLong) / 2;



//***************Find the most commonly visited street addresses

	print('<div style="position:absolute;top:0px;width:100%;">Coordinates Loaded.<br>Consolidating Duplicate Coordinates.<br>
	');
	flush();


//Grab the latest location

	$posCurr = $positionFirst;
	$addresses = array();

	$addressCurrent = new Address($posCurr);
	$posCurr->setAddress($addressCurrent);
	$addresses[$posCurr->getLat().','.$posCurr->getLong()] = $addressCurrent;

//Consolidate duplicate locations

	if(!$posCurr->isLastPosition()){
		do{
			$posCurr = $posCurr->getNextPosition();
			if(!(isset($addresses[$posCurr->getLat().','.$posCurr->getLong()]))){
				$addresses[$posCurr->getLat().','.$posCurr->getLong()] = new Address($posCurr);
				$posCurr->setAddress($addresses[$posCurr->getLat().','.$posCurr->getLong()]);
			}
			else{
				$addresses[$posCurr->getLat().','.$posCurr->getLong()]->addPosition($posCurr);
				$posCurr->setAddress($addresses[$posCurr->getLat().','.$posCurr->getLong()]);
			}

		} while(!($posCurr->isLastPosition()));
	}


//Retrieve and sort reverse-geolocated addresses
	print('Determining Address Data for '.count($addresses).' Location'.plu(count($addresses)).'.');
	flush();
	$calls = 0;

//Check the cache
	$query="select * FROM addressCache WHERE coordinates='".coordinateTruncate($addressCurrent->getLat(), $addressCurrent->getLong())."' LIMIT 1";
	$result = mysql_query($query) or die('mysql error:' . mysql_error());
	if(mysql_num_rows($result) > 0 && mysql_num_rows($result) != false){
		$row = mysql_fetch_row($result);
		$addressCurrent->setAddressString($row[1]);
		$query="update addressCache SET time=now() WHERE coordinates='".coordinateTruncate($addressCurrent->getLat(), $addressCurrent->getLong())."' LIMIT 1";
		$result = mysql_query($query) or die('mysql error:' . mysql_error());
	}
	else{
		//Process the latest location
		$handle = @fopen('http://maps.google.com/maps/api/geocode/xml?latlng='.$addressCurrent->getLat().','.$addressCurrent->getLong().'&sensor=true', "r");
		$buffer = '';
		if ($handle) {
			$buffer = fread($handle, 1000);
		}
		fclose($handle);
		if(substr($buffer, (strpos($buffer, "<status>") + 8), (strpos($buffer, "</status>") - (strpos($buffer, "<status>") + 8))) != "ZERO_RESULTS"){
			$addressCurrent->setAddressString(substr($buffer, (strpos($buffer, "<formatted_address>") + 19), (strpos($buffer, "</formatted_address>") - (strpos($buffer, "<formatted_address>") + 19))));
		}
		else{
			$addressCurrent->setAddressString(0);
		}
		$query="insert INTO addressCache (coordinates, address, time) VALUES('".coordinateTruncate($addressCurrent->getLat(), $addressCurrent->getLong())."', '".addslashes($addressCurrent->getAddressString())."', now())";
		$result = mysql_query($query) or die('mysql error:' . mysql_error());
		$calls++;
		usleep(10000);
	}

	unset($result);

//Process the most significant locations

	$addresses = array_values($addresses);

	$limit = 0;
	$i = 0;

	do{
		$addressCurr = $addresses[$i];
		$query="select * FROM addressCache WHERE coordinates='".coordinateTruncate($addressCurr->getLat(), $addressCurr->getLong())."' LIMIT 1";
		$result = mysql_query($query) or die('Error checking the cache table:' . mysql_error());
//print_r(mysql_num_rows($result));
		if(mysql_num_rows($result) > 0 && mysql_num_rows($result) != false){
//print('<br>
//FOUND<br>
//');
			$row = mysql_fetch_row($result);
//print_r($row);
			$addressCurr->setAddressString($row[1]);
			$query="update addressCache SET time=now() WHERE coordinates='".coordinateTruncate($addressCurr->getLat(), $addressCurr->getLong())."' LIMIT 1";
			$result = mysql_query($query) or die('mysql error:' . mysql_error());
		}
		elseif($addressCurr->getTimeTotal() > 750){
//print('CALL');
			//Process the location
			$handle = @fopen('http://maps.google.com/maps/api/geocode/xml?latlng='.$addressCurr->getLat().','.$addressCurr->getLong().'&sensor=true', "r");
			$buffer = '';
			if ($handle) {
				$buffer = fread($handle, 1000);
			}
			fclose($handle);
			if(($calls / 10) == round($calls / 10)){
				print('.');
			}
			if(($calls / 100) == round($calls / 100)){
				print(' ');
			}
			$calls++;
			flush();
			if(strpos($buffer, "OVER_QUERY_LIMIT") !== false){
				$limit = 1;
				break;
			}
			if(substr($buffer, (strpos($buffer, "<status>") + 8), (strpos($buffer, "</status>") - (strpos($buffer, "<status>") + 8))) != "ZERO_RESULTS"){
				$addressCurr->setAddressString(substr($buffer, (strpos($buffer, "<formatted_address>") + 19), (strpos($buffer, "</formatted_address>") - (strpos($buffer, "<formatted_address>") + 19))));
			}
			else{
				$addressCurr->setAddressString(0);
			}
			$query="insert INTO addressCache (coordinates, address, time) VALUES('".coordinateTruncate($addressCurr->getLat(), $addressCurr->getLong())."', '".addslashes($addressCurr->getAddressString())."', now())";
			$result = mysql_query($query) or die('mysql error:' . mysql_error());
			usleep(10000);
		
		}
		
		$i++;

	} while(isset($addresses[$i]));

	unset($result);
/*
$i=0;
print('
<br>
');
while($addresses[$i]){
print_r($addresses[$i]->getAddressString());
print('<br>
');
$i++;
}
*/

	if($limit == 0){

		$query = "delete FROM addressCache WHERE time < date_sub(now(), interval 1 week);";
		$result = mysql_query($query) or die('Error cleaning the cache:' . mysql_error());

//Consolidate multiple coordinates that are found at the same address
//Also correct some next-door errors due to GPS inaccuracies where possible.

		function sortAddressesByString($a, $b){

//print(strtolower($a->getAddressString()).' -&- '.strtolower($b->getAddressString()).'<br>
//');

			//$al = strrev(strtolower($a->getAddressString()));
			//$bl = strrev(strtolower($b->getAddressString()));

			//$a0 = $a->getAddressString();
			//$b0 = $b->getAddressString();

			//$al = strtolower($a0);
			//$bl = strtolower($b0);

			$a1 = $a->getAddressString();
			$b1 = $b->getAddressString();

			$a2 = join('',array_reverse(explode(',',$a1)));
			$b2 = join('',array_reverse(explode(',',$b1)));

// print($a1.' -&- '.$b1.'<br>
// ');
//print($a1.' -to- '.$a2.'<br>
//');
//print($b1.' -to- '.$b2.'<br>
//');

			if ($a2 === $b2) {
				return 0;
			}
			return ($a2 > $b2) ? +1 : -1;
		}

		usort($addresses, "sortAddressesByString");

/*
//print_r($addresses);
$i=0;
print('
<br>
');
while($addresses[$i]){
print($addresses[$i]->getAddressString().'<br>
');
$i++;
}
*/

		$addressPrev = reset($addresses);
		$addressCurr = next($addresses);
		$addressNew = null;

		if(count($addresses) > 1){
			do{
				if($addressPrev->getAddressString()){

					//If there's an address consolidation in progress
					if(isset($addressNew)){
						if(($addressNew->getAddressString() == $addressCurr->getAddressString()) 
						    || ((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < $addressNew->getAcc()) && (getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < $addressCurr->getAcc()))
						    || ((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < 65) && ($addressNew->getAcc() < 5000) && ($addressCurr->getAcc() < 5000))){
							if($addressCurr->getTimeTotal() >= $addressNew->getTimeTotal()){
								$positionsOld = $addressNew->getPositions();
								$addressCurr->addPositions($positionsOld);
								$i = 0;
								while(isset($positionsOld[$i])){
									$positionsOld[$i]->setAddress($addressCurr);
									$i++;
								}
								$addressNew = $addressCurr;
							}
							else{
								$positionsOld = $addressCurr->getPositions();
								$addressNew->addPositions($positionsOld);
								$i = 0;
								while(isset($positionsOld[$i])){
									$positionsOld[$i]->setAddress($addressNew);
									$i++;
								}
							}
						}
						else{
							$addresses2[] = $addressNew;
							$addressNew = null;
						}
					}
					elseif(($addressPrev->getAddressString() == $addressCurr->getAddressString())
					    || ((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < $addressPrev->getAcc()) && (getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < $addressCurr->getAcc()))
					    || ((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < 65) && ($addressPrev->getAcc() < 5000) && ($addressCurr->getAcc() < 5000))){
						if($addressCurr->getTimeTotal() >= $addressPrev->getTimeTotal()){
							$positionsOld = $addressPrev->getPositions();
							$addressCurr->addPositions($positionsOld);
							$i = 0;
							while(isset($positionsOld[$i])){
								$positionsOld[$i]->setAddress($addressCurr);
								$i++;
							}
							$addressNew = $addressCurr;
						}
						else{
							$positionsOld = $addressCurr->getPositions();
							$addressPrev->addPositions($positionsOld);
							$i = 0;
							while(isset($positionsOld[$i])){
								$positionsOld[$i]->setAddress($addressPrev);
								$i++;
							}
							$addressNew = $addressPrev;
						}
					}
					else{
						$addresses2[] = $addressPrev;
					}
				}

				$addressPrev = $addressCurr;
				$addressCurr = next($addresses);

			} while($addressCurr);

			if(isset($addressNew)){
				$addresses2[] = $addressNew;
			}
			elseif($addresses2[count($addresses2) - 1] !== $addressPrev){
				$addresses2[] = $addressPrev;
			}
		}
		else{
			$addresses2 = $addresses;
		}

		unset($addresses);
		$addresses = $addresses2;
		unset($addresses2);

/*
$i=0;
print('
<br>
');
while($addresses[$i]){
print($addresses[$i]->getAddressString().'<br>
');
$i++;
}
*/

//Consolidate multiple coordinates that are found at similar addresses
//Also correct more next-door errors due to GPS inaccuracies where possible.


		function sortAddressesByString2($a, $b){

			$a1 = $a->getSortString();
			$b1 = $b->getSortString();
//print($a1." -&- ".$b1."<br>
//");
			if ($a1 === $b1) {
//print 'Found';
				return 0;
			}


			//If the street is the same, sort by total time spent
			$a2 = explode(" ",$a1);
			$b2 = explode(" ",$b1);
			$a3 = array_pop($a2);
			//$a3 = array_pop($a2);
//print($a3.' is a3 ');
			$b3 = array_pop($b2);
			//$b3 = array_pop($b2);
//print($b3.' is b3
//');
//flush();

			if((join('',$a2) === join('',$b2)) && is_numeric($a3) && is_numeric($b3)){
				return ($a->getTimeTotal() < $b->getTimeTotal()) ? +1 : -1;
			}
			
//print(strcasecmp((string)$al, (string)$bl));
//print($al === $bl);
//print($a->getSortString() > $b->getSortString());
			//Else, sort by street
			return ($a->getSortString() > $b->getSortString()) ? +1 : -1;
		}

		function isCloseAddress($address1, $address2){
//print('<br>
//'.$address1->getAddressString().' *** '.$address2->getAddressString().' *** ');
//flush();
//print(' '.$num1.' '.$num2.' ');
//			if($address1->getAddressString() == $address2->getAddressString()){
//print('Yes0');
//				return true;
//			}
			$num1 = current(explode(' ', $address1->getAddressString()));
			$num2 = current(explode(' ', $address2->getAddressString()));

			if(!is_numeric($num1)){
//print('No1');
				return false;
			}
			if(!is_numeric($num2)){
//print('No2');
				return false;
			}
			if(abs($num1 - $num2) < ($num1 / 75)){
//print('Yes1');
				return true;
			}
			if(abs($num1 - $num2) < ($num2 / 75)){
//print('Yes2');
				return true;
			}
//print('No3');
			return false;
		}

		usort($addresses, "sortAddressesByString2");

//$addressCurr = reset($addresses);
//while($addressCurr){
//print('
//<br>'.$addressCurr->getAddressString());
//$addressCurr = next($addresses);
//}

		$addressPrev = reset($addresses);
		$addressCurr = next($addresses);
		$addressNew = null;

		if(count($addresses) > 1){
			do{
				if($addressPrev->getAddressString() != ""){

				//If there's an address consolidation in progress
				if(isset($addressNew)){
					if(isCloseAddress($addressCurr,$addressNew) && (((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < $addressNew->getAcc()) && (getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < $addressCurr->getAcc()))
					  || (($addressNew->getMethod() > 1) && ($addressCurr->getMethod() > 1) && getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressNew->getLat(),$addressNew->getLong()) < 65))){
						if($addressCurr->getTimeTotal() >= $addressNew->getTimeTotal()){
							$positionsOld = $addressNew->getPositions();
							$addressCurr->addPositions($positionsOld);
							$i = 0;
							while(isset($positionsOld[$i])){
								$positionsOld[$i]->setAddress($addressCurr);
								$i++;
							}
							$addressNew = $addressCurr;
						}
						else{
							$positionsOld = $addressCurr->getPositions();
							$addressNew->addPositions($positionsOld);
							$i = 0;
							while(isset($positionsOld[$i])){
								$positionsOld[$i]->setAddress($addressNew);
								$i++;
							}
						}
					}
					else{
						$addresses2[] = $addressNew;
						$addressNew = null;
					}
				}
				elseif(isCloseAddress($addressCurr,$addressPrev) && (((getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < $addressPrev->getAcc()) && (getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < $addressCurr->getAcc()))
					  || (($addressPrev->getMethod() > 1) && ($addressCurr->getMethod() > 1) && getDistance($addressCurr->getLat(),$addressCurr->getLong(),$addressPrev->getLat(),$addressPrev->getLong()) < 65))){
					if($addressCurr->getTimeTotal() >= $addressPrev->getTimeTotal()){
						$positionsOld = $addressPrev->getPositions();
						$addressCurr->addPositions($positionsOld);
						$i = 0;
						while(isset($positionsOld[$i])){
							$positionsOld[$i]->setAddress($addressCurr);
							$i++;
						}
						$addressNew = $addressCurr;
					}
					else{
						$positionsOld = $addressCurr->getPositions();
						$addressPrev->addPositions($positionsOld);
						$i = 0;
						while(isset($positionsOld[$i])){
							$positionsOld[$i]->setAddress($addressPrev);
							$i++;
						}
						$addressNew = $addressPrev;
					}
				}
				else{
					$addresses2[] = $addressPrev;
				}
				}

				$addressPrev = $addressCurr;
				$addressCurr = next($addresses);

			} while($addressCurr);

			if(isset($addressNew)){
				$addresses2[] = $addressNew;
			}
			elseif($addresses2[count($addresses2) - 1] !== $addressPrev){
				$addresses2[] = $addressPrev;
			}
		}
		else{
			$addresses2 = $addresses;
		}

		$addresses3 = $addresses2;
		unset($addresses2);
		unset($addresses);


		print('<br>'.count($addresses3).' Distinct Addresses Found.<br>Sorting Addresses by Time Stayed.<br>');

		flush();

		unset($addresses2);

//Sort addresses by time spent and remove addresses with no stays or that are egregiously inaccurate.
//Also, check the cache and correct address strings where possible.

		$i = 0;

		while(isset($addresses3[$i])){

			$query="select * FROM addressCache WHERE coordinates='".coordinateTruncate($addresses3[$i]->getLat(), $addresses3[$i]->getLong())."' LIMIT 1";
			$result = mysql_query($query) or die('mysql error:' . mysql_error());
			if(mysql_num_rows($result) > 0 && mysql_num_rows($result) != false){
				$row = mysql_fetch_row($result);
				$addresses3[$i]->setAddressString($row[1]);
			}
			elseif($limit == 0){
				//Process the location
				$handle = @fopen('http://maps.google.com/maps/api/geocode/xml?latlng='.$addresses3[$i]->getLat().','.$addresses3[$i]->getLong().'&sensor=true', "r");
				$buffer = '';
				if ($handle) {
					$buffer = fread($handle, 1000);
				}
				fclose($handle);
				$calls++;
				if(strpos($buffer, "OVER_QUERY_LIMIT") !== false){
					$limit = 1;
					continue;
				}
				if(substr($buffer, (strpos($buffer, "<status>") + 8), (strpos($buffer, "</status>") - (strpos($buffer, "<status>") + 8))) != "ZERO_RESULTS"){
					$addresses3[$i]->setAddressString(substr($buffer, (strpos($buffer, "<formatted_address>") + 19), (strpos($buffer, "</formatted_address>") - (strpos($buffer, "<formatted_address>") + 19))));
				}
				$query="insert INTO addressCache (coordinates, address, time) VALUES('".coordinateTruncate($addresses3[$i]->getLat(), $addresses3[$i]->getLong())."', '".$addresses3[$i]->getAddressString()."', now())";
				$result = mysql_query($query) or die('mysql error:' . mysql_error());
				usleep(10000);
			}

			if(($addresses3[$i]->getTimeTotal() > 0) && ($addresses3[$i]->getAcc() < 1000)){
				$addressesByTime[$addresses3[$i]->getTimeTotal()] = $addresses3[$i];
			}
			$i++;
		}

		unset($result);
		unset($addresses3);

		if(isset($addressesByTime)){
			krsort($addressesByTime);
		}
		print($calls.' Google Geocoding Call'.plu($calls).' Used.<br>Correcting Address Errors.<br>
		');
	}
	else{
		print('<br>Geolocation query limit reached.<br>');
	}


//***************Draw the map

	print('Drawing Map...</div>');
	flush();
	print('<script type="text/javascript">


		var addressConsolidated = [];
		var totalTime = 0;

		function codeLatLng(latlng) {
			var geocoder = new google.maps.Geocoder();
			geocoder.geocode({\'latLng\': latlng}, function(results, status) {
				if (status == google.maps.GeocoderStatus.OK) {
					if (results[1]) {
						return results[1].formatted_address;
					}
					else{
						return 0; 
					}
        			}
				else {
					alert("Geocoder failed due to: " + status);
				}
			});
  		}

		function printTable() {
			if(addressConsolidated.length > 0){
				document.write ("<table width=\"100%\" cellspacing=0 style=\"border-top-width:1px;border-top-style:solid;border-top-color:#808080;margin-top:20px;\"><tr style=\"font-weight:bold;\"><td>Frequently-Visited Addresses</td></tr><tr><td>" +  addressConsolidated[0][0] + "</td></tr>");
				if(addressConsolidated.length == 2){
					document.write ("<tr><td>" +  addressConsolidated[1][0] + "</td></tr>");
				}
				if(addressConsolidated.length == 3){
					document.write ("<tr><td>" +  addressConsolidated[2][0] + "</td></tr>");
				}
				document.write ("</table>");
			}
		}

		function initialize() {
			var latlng = new google.maps.LatLng('.$centerLat.', '.$centerLong.');
			var myOptions = {
				//zoom: 15,
				//center: latlng,
				mapTypeId: google.maps.MapTypeId.ROADMAP
			};
			var map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);');

			if(!$positionFirst->isLastPosition()){

			print('var pathCoordinates = [');

				$posCurr = $positionFirst;

				do{
					print('new google.maps.LatLng('.$posCurr->getLat().', '.$posCurr->getLong().')');
					print(',');
					$posCurr = $posCurr->getNextPosition();
					if($posCurr->isLastPosition()){
						print('new google.maps.LatLng('.$posCurr->getLat().', '.$posCurr->getLong().')');
					}
				} while(!($posCurr->isLastPosition()));
				print(']

			');

			if($positionCount > 1){
				print('

			var travelPath = new google.maps.Polyline({
				path: pathCoordinates,
				strokeColor: "#0000FF",
				strokeOpacity: 0.5,
				strokeWeight: ');
				if($positionCount < 100){
					print('3');
				}
				elseif($positionCount < 1000){
					print('2');
				}
				else{
					print('1');
				}
				print('
			});

			travelPath.setMap(map);

				');
			}
			}
			print('

			//  Make an array of the LatLngs of the markers you want to show
			//var LatLngList = array (new google.maps.LatLng ('.$maxLat.','.$maxLong.'), new google.maps.LatLng ('.$minLat.','.$minLong.'));
			//  Create a new viewpoint bound
			var bounds = new google.maps.LatLngBounds ();');

			if(!$positionFirst->isLastPosition()){
				print('
				//  Go through each...
				for (var i = 0, LtLgLen = pathCoordinates.length; i < LtLgLen; i++) {
					//  And increase the bounds to take this point
					bounds.extend (pathCoordinates[i]);
				}');
			}

			print('var latlngacc1 = new google.maps.LatLng('.($positionFirst->getLat() + ($positionFirst->getAcc() * 0.0000089928)).','.$positionFirst->getLong().');
			var latlngacc2 = new google.maps.LatLng('.($positionFirst->getLat() - ($positionFirst->getAcc() * 0.0000089928)).','.$positionFirst->getLong().');

			bounds.extend (latlngacc1);
			bounds.extend (latlngacc2);

			//  Fit these bounds to the map
			map.fitBounds (bounds);');

			function printMarker($address, $color, $visible){
				print('var imagef = \'http://www.google.com/intl/en_us/mapfiles/ms/micons/'.$color.'.png\';
				var latlngf = new google.maps.LatLng('.$address->getLat().', '.$address->getLong().');
				var markerf = new google.maps.Marker({
					position: latlngf, 
					map: map, 
					title:"'.$address->getAddressString().'",
					icon: imagef,');
					if($color == "red-dot"){
						$color = "red";
						print('zIndex: 10,');
					}
					else{
						print('zIndex: 5,');
					}
					if($visible){
						print('visible: true');
					}
					else{
						print('visible: false');
					}
				print('});');
				if($visible){
					print('var currCircle = new google.maps.Circle({
						radius: '.$address->getAcc().',
                       	       	center: latlngf,
                       	       	strokeColor: "'.$color.'",
                       	       	strokeOpacity: 0.5,
                       	       	strokeWeight: 2,
                       	       	fillColor: "'.$color.'",
                        	       	fillOpacity: 0.25,
						map: map
					});');
				}
			}

			$found = false;

			if($limit == 0){
				if(count($addressesByTime) > 0){

			$addressesToPrint[0] = $positionFirst->getAddress();

			$addressCurr = reset($addressesByTime);
			if($addressCurr && (/*$addressCurr->getAddressString() == $positionFirst->getAddress()->getAddressString() || */$addressCurr == $positionFirst->getAddress())){
				$addressesToPrint[0] = $addressCurr;
				$addressCurr = next($addressesByTime);
			}
			if($addressCurr){
				$addressesToPrint[] = $addressCurr;
				$address1 = $addressCurr;
			}
			$addressCurr = next($addressesByTime);
			if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
				$addressesToPrint[0] = $addressCurr;
				$addressCurr = next($addressesByTime);
			}
			if($addressCurr){
				$addressesToPrint[] = $addressCurr;
				$address2 = $addressCurr;
			}
			$addressCurr = next($addressesByTime);
			if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
				$addressesToPrint[0] = $addressCurr;
				$addressCurr = next($addressesByTime);
			}
			if($addressCurr){
				$addressesToPrint[] = $addressCurr;
				$address3 = $addressCurr;
			}
			$addressCurr = next($addressesByTime);
			if($addressCurr){
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
				$addressCurr = next($addressesByTime);
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
				$addressCurr = next($addressesByTime);
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
				$addressCurr = next($addressesByTime);
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
				$addressCurr = next($addressesByTime);
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
				$addressCurr = next($addressesByTime);
				if($addressCurr && ($addressCurr == $positionFirst->getAddress())){
					$addressesToPrint[0] = $addressCurr;
					$addressCurr = next($addressesByTime);
				}
				if($addressCurr){
					$addressesToPrint[] = $addressCurr;
				}
			}

			printMarker($addressesToPrint[0], "red-dot", true);

			if(isset($addressesToPrint[1])){
				printMarker($addressesToPrint[1], "pink", true);
			}
			if(isset($addressesToPrint[2])){
				printMarker($addressesToPrint[2], "purple", true);
			}
			if(isset($addressesToPrint[3])){
				printMarker($addressesToPrint[3], "blue", true);
			}


				}
				else{
					printMarker($positionFirst->getAddress(), "red-dot", true);
				}
			}

		print('}

	</script>');

	print('<div id="map_canvas" style="width:100%; height:640px; border-width:0px; margin:0px; padding:0px;"></div><br><center>');
	print('<form action="" method="GET">'.intervalField($amnt).nameField().'<input type="submit" value="OK"></form>');


//Display the address table 

	if($limit == 0){

		function printAddressEntry($address, $color){
			print('<tr style="background-color:'.$color.';"><td style="width:69%;">');
			print('<a href="http://maps.google.com/maps?q='.urlencode($address->getAddressString()).'" target="_BLANK">'.htmlspecialchars_decode(htmlentities($address->getAddressString(),ENT_COMPAT,"UTF-8")).'</a>');
//print($address->getLat().', '.$address->getLong());
			print('</td><td style="width:20%;">');
			$spent = datetime_diff($address->getTimeTotal(), 0);
			if($spent["days_total"] > 0){
				print($spent["days_total"].' day'.plu($spent["days_total"]).', '.$spent["hours"].' hour'.plu($spent["hours"]).', '.$spent["minutes"].' minute'.plu($spent["minutes"]).'');
			}
			elseif($spent["hours_total"] > 0){
				print($spent["hours_total"].' hour'.plu($spent["hours_total"]).', '.$spent["minutes"].' minute'.plu($spent["minutes"]).'');
			}
			else{
				print($spent["minutes"].' minute'.plu($spent["minutes"]));
			}
			print('</td><td>'.round($address->getAcc(),2).'m</td></tr>');
		}

		print('<table width="100%" cellspacing=0 style="border-top-width:1px;border-top-style:solid;border-top-color:#808080;margin-top:10px;"><tr style="font-weight:bold;"><td>Latest Known Address</td><td>Known Time Spent</td><td>Accuracy</td></tr>');

		if(count($addressesByTime) > 0){

			printAddressEntry($addressesToPrint[0], "#FFE6E6");

			if(isset($addressesToPrint[1])){
				print('<tr style="font-weight:bold;"><td>Other Notable Addresses</td><td>&nbsp;</td><td>&nbsp;</td></tr>');
				printAddressEntry($addressesToPrint[1], "#FBE6F6");
			}
			if(isset($addressesToPrint[2])){
				printAddressEntry($addressesToPrint[2], "#F0E6FD");
			}
			if(isset($addressesToPrint[3])){
				printAddressEntry($addressesToPrint[3], "#E6EcFF");
			}
			print('</table>');


			if(isset($addressesToPrint[4])){
				print('<div style="width:100%;text-align:left;"><a href="javascript:ReverseContentDisplay(\'Addresses\')">Show/Hide More Addresses</a></div><div id="Addresses" style="display:none;"><table width="100%" cellspacing=0 style="border-top-width:1px;border-top-style:solid;border-top-color:#808080;">');
			}

			$i = 4;
			while(isset($addressesToPrint[$i]) && ($i < 10)){
				printAddressEntry($addressesToPrint[$i], "#FFFFFF");
				$i++;
			}

			if(isset($addressesToPrint[4])){
				print('</table></div>');
			}

		}
		else{
			printAddressEntry($positionFirst->getAddress(), "#FFE6E6");
			print('</table>');
		}

		print('<br>');
		

	}
	else{
		print('<div style="color:#800000;font-weight:bold;">Address Identification Query Limit Reached.<br>Street Addresses Cannot Be Identified.</div><br>');
	}

//Display the Coordinate Table

	print('<div style="display:inline;margin:0px;padding:0px;width:40px;height:20px;border-width:1px;border-style:solid;border-color:#808080;background-color:#FFE4E4;">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</div> Latest Location &nbsp;&nbsp;&nbsp;<div style="display:inline;margin:0px;padding:0px;width:13px;height:20px;border-width:1px;border-style:solid;border-color:#808080;background-color:#FFE6F4;">&nbsp;&nbsp;&nbsp;</div><div style="display:inline;margin:0px;padding:0px;width:13px;height:20px;border-width:1px;border-left-width:0px;border-right-width:0px;border-style:solid;border-color:#808080;background-color:#EcE6FF;">&nbsp;&nbsp;&nbsp;</div><div style="display:inline;margin:0px;padding:0px;width:13px;height:20px;border-width:1px;border-style:solid;border-color:#808080;background-color:#E6EcFF;">&nbsp;&nbsp;&nbsp;</div> Notable Locations &nbsp;&nbsp;&nbsp;<div style="display:inline;margin:0px;padding:0px;width:40px;height:20px;border-width:1px;border-style:solid;border-color:#808080;background-color:#ECECEC;">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</div> Standard</center>');
	print('<table width="100%" cellspacing=0 style="border-top-width:1px;border-top-style:solid;border-top-color:#808080;margin-top:20px;"><tr style="font-weight:bold;"><td>Time</td><td>Stayed</td><td>Coordinates</td><td>Altitude</td><td>Accuracy</td><td>Battery</td></tr>');

	$posCurr = $positionFirst;
	$posPrev = null;
	$gap = 0;

	$color = "#FFE6E6";
	do{
			if(isset($posPrev)){
				$gap = datetime_diff($posCurr->getLatestTime(), $posPrev->getEarliestTime());
			}
			// Check for and display gaps in the coordinates
			if((isset($posPrev) && $interval != "" && $gap["seconds_total"] > ($interval + 900))){
				print('<tr style="background-color:#FFFFFF"><td colspan="6">');
				if($gap["days_total"] > 0){
					print($gap["days_total"].' day, '.$gap["hours"].' hour, '.$gap["minutes"].' minute gap - ');
				}
				elseif($gap["hours_total"] > 0){
					print($gap["hours_total"].' hour, '.$gap["minutes"].' minute gap - ');
				}
				else{
					print($gap["minutes"].' minute gap - ');
				}
				if($posCurr->getLatestRecordIndex() != ($posPrev->getEarliestRecordIndex() + 1)){
					print('reported coordinates not sufficiently accurate.');
				}
				else{
					print('no coordinates reported.');
				}
				print('</td></tr>');
				if($color != "#E2E2E2"){
					$color = "#E2E2E2";
				}
				else{
					$color = "#ECECEC";
				}
			}
			if($posCurr->getAddress() === $address1){
				$color = "#FBE6F6";
			}
			elseif($posCurr->getAddress() === $address2){
				$color = "#F0E6FD";
			}
			elseif($posCurr->getAddress() === $address3){
				$color = "#E6EcFF";
			}
			elseif($posCurr->getAddress() === $positionFirst->getAddress()){
				$color = "#FFE6E6";
			}
			$dateArray = datetime_diff($posCurr->getEarliestTime(), $posCurr->getLatestTime());
			if(($posCurr->getEarliestTime() != $posCurr->getLatestTime()) && $dateArray["minutes_total"] > 1){
				if($dateArray["minutes_total"] > 59){
					print('<tr style="background-color:'.$color.';"><td>'.date("M j, Y g:i A",strtotime($posCurr->getEarliestTime())).' -<br>'.date("M j, Y g:i A",strtotime($posCurr->getLatestTime())).'</td><td>'.$dateArray["hours_total"].' hour'.plu($dateArray["hours_total"]).',<br>'.$dateArray["minutes"].' minute'.plu($dateArray["minutes"]).'</td>');
				}
				else{
					print('<tr style="background-color:'.$color.';"><td>'.date("M j, Y g:i A",strtotime($posCurr->getLatestTime())).' -<br>'.date("M j, Y g:i A",strtotime($posCurr->getEarliestTime())).'</td><td>'.$dateArray["minutes_total"].' minute'.plu($dateArray["minutes_total"]).'</td>');
				}
			}
			else{
				print('<tr style="background-color:'.$color.';"><td>'.date("M j, Y g:i A",strtotime($posCurr->getEarliestTime())).'</td><td>--</td>');
			}
			if($posCurr->getAlt() == 0){
				$altString = "--";
			}
			else{
				$altString = round($posCurr->getAlt())."m"; 
			}
			print('<td><a href="http://maps.google.com/maps?q='.$posCurr->getLat().','.$posCurr->getLong().'" target="_BLANK">'.round($posCurr->getLat(), 4).', '.round($posCurr->getLong(), 4).'</a> ');
//print(' '.$posCurr->getMethod());
			print('</td><td>'.$altString.'</td><td>'.round($posCurr->getAcc(), 2).'m</td>');
			if($posCurr->getLowestBatt() < 0){
				if($posCurr->getHighestBatt() < 0){
					print('<td>--</td>');
				}
				else{
					print('<td>'.$posCurr->getHighestBatt().'%</td>');
				}
			}
			elseif($posCurr->getHighestBatt() < 0){
				print('<td>'.$posCurr->getLowestBatt().'%</td>');
			}
			elseif($posCurr->getLowestBatt() != $posCurr->getHighestBatt()){
				print('<td>'.$posCurr->getLowestBatt().'% - '.$posCurr->getHighestBatt().'%</td>');
			}
			else{
				print('<td>'.$posCurr->getLowestBatt().'%</td>');
			}
			if($color != "#E2E2E2"){
				$color = "#E2E2E2";
			}
			else{
				$color = "#ECECEC";
			}

			print('</tr>');

			$posPrev = $posCurr;
			$posCurr = $posCurr->getNextPosition();
			
	} while(isset($posCurr));

	print('</table>');

	print('</body></html>');

}

mysql_close();

}

?>

